import json_pbp
import html_pbp
import espn_pbp
import json_shifts
import html_shifts
import playing_roster
import json_schedule
import pandas as pd
import time
import datetime
import shared

# Holds list for broken games for shifts and pbp
broken_shifts_games = []
broken_pbp_games = []
players_missing_ids = []
espn_games = []

pbp_columns = ['Game_Id', 'Date', 'Period', 'Event', 'Description', 'Time_Elapsed', 'Seconds_Elapsed', 'Strength',
           'Ev_Zone', 'Type', 'Ev_Team', 'Home_Zone', 'Away_Team', 'Home_Team', 'p1_name', 'p1_ID', 'p2_name', 'p2_ID',
           'p3_name', 'p3_ID', 'awayPlayer1', 'awayPlayer1_id', 'awayPlayer2', 'awayPlayer2_id', 'awayPlayer3',
           'awayPlayer3_id', 'awayPlayer4', 'awayPlayer4_id', 'awayPlayer5', 'awayPlayer5_id', 'awayPlayer6',
           'awayPlayer6_id', 'homePlayer1', 'homePlayer1_id', 'homePlayer2', 'homePlayer2_id', 'homePlayer3',
           'homePlayer3_id', 'homePlayer4', 'homePlayer4_id', 'homePlayer5', 'homePlayer5_id', 'homePlayer6',
           'homePlayer6_id',  'Away_Players', 'Home_Players', 'Away_Score', 'Home_Score', 'Away_Goalie',
           'Away_Goalie_Id', 'Home_Goalie', 'Home_Goalie_Id', 'xC', 'yC', 'Home_Coach', 'Away_Coach']


def check_goalie(row):
    """
    Checks for bad goalie names (you can tell by them having no player id)
    :param row: df row
    """
    if row['Away_Goalie'] != '' and row['Away_Goalie_Id'] == 'NA':
        players_missing_ids.extend([row['Away_Goalie'], row['Game_Id']])

    if row['Home_Goalie'] != '' and row['Home_Goalie_Id'] == 'NA':
        players_missing_ids.extend([row['Home_Goalie'], row['Game_Id']])


def get_players_json(json):
    """
    Return dict of players for that game
    :param json: gameData section of json
    :return: dict of players->keys are the name (in uppercase)  
    """
    players = dict()

    players_json = json['players']
    for key in players_json.keys():
        name = shared.fix_name(players_json[key]['fullName'].upper())
        players[name] = {'id': ' '}
        try:
            players[name]['id'] = players_json[key]['id']
        except KeyError:
            print(name, ' is missing an ID number')
            players[name]['id'] = 'NA'

    return players


def combine_players_lists(json_players, roster_players, game_id):
    """
    Combine the json list of players (which contains id's) with the list in the roster html
    :param json_players: dict of all players with id's
    :param roster_players: dict with home and and away keys for players
    :param game_id: id of game
    :return: dict containing home and away keys -> which contains list of info on each player
    """
    home_players = dict()
    for player in roster_players['Home']:
        try:
            name = shared.fix_name(player[2])
            id = json_players[name]['id']
            home_players[name] = {'id': id, 'number': player[0]}
        except KeyError:
            # This usually means it's the backup goalie (who didn't play) so it's no big deal with them
            if player[1] != 'G':
                players_missing_ids.extend([player, game_id])
                home_players[name] = {'id': 'NA', 'number': player[0]}

    away_players = dict()
    for player in roster_players['Away']:
        try:
            name = shared.fix_name(player[2])
            id = json_players[name]['id']
            away_players[name] = {'id': id, 'number': player[0]}
        except KeyError:
            if player[1] != 'G':
                players_missing_ids.extend([player, game_id])
                away_players[name] = {'id': 'NA', 'number': player[0]}

    return {'Home': home_players, 'Away': away_players}


def combine_html_json_pbp(json_df, html_df, game_id, date):
    """
    Join both data sources
    :param json_df: json pbp DataFrame
    :param html_df: html pbp DataFrame
    :param game_id: id of game
    :param date: date of game
    :return: finished pbp
    """
    try:
        html_df.Period = html_df.Period.astype(int)
        game_df = pd.merge(html_df, json_df, left_on=['Period', 'Event', 'Seconds_Elapsed'],
                           right_on=['period', 'event', 'seconds_elapsed'], how='left')

        # This id because merge doesn't work well with shootouts
        game_df = game_df.drop_duplicates(subset=['Period', 'Event', 'Description', 'Seconds_Elapsed'])
        game_df['Game_Id'] = game_id[-5:]
        game_df['Date'] = date
        return pd.DataFrame(game_df, columns=pbp_columns)
    except Exception as e:
        print('Problem combining Html Json pbp for game {}'.format(game_id, e))


def combine_espn_html_pbp(html_df, espn_df, game_id, date, away_team, home_team):
    """
    Merge the coordinate from the espn feed into the html DataFrame
    :param html_df: dataframe with info from html pbp
    :param espn_df: dataframe with info from espn pbp
    :param game_id: json game id
    :param date: ex: 2016-10-24
    :param away_team: away team
    :param home_team: home team
    :return: merged DataFrame
    """
    try:
        espn_df.period = espn_df.period.astype(int)
        df = pd.merge(html_df, espn_df, left_on=['Period', 'Seconds_Elapsed', 'Event'],
                      right_on=['period', 'time_elapsed', 'event'], how='left')

        # df = df.drop_duplicates(subset=['Period', 'Event', 'Seconds_Elapsed'])
        df = df.drop(['period', 'time_elapsed', 'event'], axis=1)
    except Exception as e:
        print('Error for combining espn and html pbp for game {}'.format(game_id), e)
        return None

    df['Game_Id'] = game_id[-5:]
    df['Date'] = date
    df['Away_Team'] = away_team
    df['Home_Team'] = home_team

    return pd.DataFrame(df, columns=pbp_columns)


def scrape_pbp(game_id, date, roster):
    """
    Scrapes the pbp
    Automatically scrapes the json and html, if the json is empty the html picks up some of the slack and the espn
    xml is also scraped for coordinates
    :param game_id: json game id
    :param date: date of game
    :param roster: list of players in pre game roster
    :return: 1. DataFrame with info or None if it fails
             2. a dict of players with id's and numbers
    """
    game_json = json_pbp.get_pbp(game_id)
    try:
        teams = json_pbp.get_teams(game_json)                                    # Get teams from json
        player_ids = get_players_json(game_json['gameData'])
        players = combine_players_lists(player_ids, roster['players'], game_id)  # Combine roster names with player id's
    except Exception as e:
        print('Problem with getting the teams or players', e)
        return None, None

    year = str(game_id)[:4]
    # Coordinates are only available in json from 2010 onwards
    if int(year) >= 2010:
        try:
            json_df = json_pbp.parse_json(game_json)
            num_json_plays = len(game_json['liveData']['plays']['allPlays'])
        except Exception as e:
            print('Error for Json pbp for game {}'.format(game_id), e)
            return None, None
    else:
        num_json_plays = 0

    # Check if the json is missing the plays...if it is enable the HTML parsing to do more work to make up for the
    # json and scrape ESPN for the coordinates
    if num_json_plays == 0:
        espn_games.extend([game_id])
        html_df = html_pbp.scrape_game(game_id, players, teams, False)
        espn_df = espn_pbp.scrape_game(date, teams['Home'], teams['Away'])
        game_df = combine_espn_html_pbp(html_df, espn_df, str(game_id), date, teams['Away'], teams['Home'])
    else:
        html_df = html_pbp.scrape_game(game_id, players, teams, True)
        game_df = combine_html_json_pbp(json_df, html_df, str(game_id), date)

    if game_df is not None:
        game_df['Home_Coach'] = roster['head_coaches']['Home']
        game_df['Away_Coach'] = roster['head_coaches']['Away']

    return game_df, players


def scrape_shifts(game_id, players, date):
    """
    Scrape the Shift charts (or TOI tables)
    :param game_id: json game id
    :param players: dict of players with numbers and id's
    :param date: date of game
    :return: DataFrame with info or None if it fails
    """
    year = str(game_id)[:4]
    try:
        if int(year) < 2010:   # Control for fact that shift json is only available from 2010 onwards
            raise Exception
        shifts_df = json_shifts.scrape_game(game_id)
    except Exception:
        try:
            shifts_df = html_shifts.scrape_game(game_id, players)
        except Exception as e:
            broken_shifts_games.extend([game_id])
            print('Error for html shifts for game {}'.format(game_id), e)
            return None

    shifts_df['Date'] = date

    return shifts_df


def scrape_game(game_id, date, if_scrape_shifts):
    """
    This scrapes the info for the game.
    The pbp is automatically scraped, and the whether or not to scrape the shifts is left up to the user
    :param game_id: game to scrap
    :param date: ex: 2016-10-24
    :param if_scrape_shifts: boolean, check if scrape shifts
    :return: DataFrame of pbp info
             (optional) DataFrame with shift info
    """

    print(' '.join(['Scraping Game ', game_id, date]))
    shifts_df = None

    try:
        roster = playing_roster.scrape_roster(game_id)
    except Exception:
        broken_pbp_games.extend([game_id, date])
        return None, None     # Everything fails without the roster

    pbp_df, players = scrape_pbp(game_id, date, roster)

    if pbp_df is None:
        broken_pbp_games.extend([game_id, date])

    if if_scrape_shifts and pbp_df is not None:
        shifts_df = scrape_shifts(game_id, players, date)

    return pbp_df, shifts_df


def scrape_list_of_games(games, if_scrape_shifts):
    """
    Given a list of game_id's (and a date for each game) it scrapes them
    :param games: list of [game_id, date]
    :param if_scrape_shifts: whether to scrape shifts
    :return: DataFrame of pbp info, also shifts if specified
    """
    pbp_dfs = []
    shifts_dfs = []

    for game in games:
        pbp_df, shifts_df = scrape_game(str(game[0]), game[1], if_scrape_shifts)
        if pbp_df is not None:
            pbp_dfs.extend([pbp_df])
        if shifts_df is not None:
            shifts_dfs.extend([shifts_df])

    # Check if any games
    if len(pbp_dfs) == 0:
        return None, None

    pbp_df = pd.concat(pbp_dfs)
    pbp_df = pbp_df.reset_index(drop=True)
    pbp_df.apply(lambda row: check_goalie(row), axis=1)

    if if_scrape_shifts:
        shifts_df = pd.concat(shifts_dfs)
        shifts_df = shifts_df.reset_index(drop=True)
    else:
        shifts_df = None

    return pbp_df, shifts_df


def scrape_date_range(from_date, to_date, if_scrape_shifts):
    """
    Scrape games in given date range
    :param from_date: date you want to scrape from
    :param to_date: date you want to scrape to
    :param if_scrape_shifts: boolean, check if scrape shifts
    """
    try:
        if time.strptime(to_date, "%Y-%m-%d") < time.strptime(from_date, "%Y-%m-%d"):
            print("Error: The second date input is earlier than the first one")
            return
    except ValueError:
        print("Incorrect format given for dates. They must be given like '2016-10-01' ")
        return

    games = json_schedule.scrape_schedule(from_date, to_date)
    pbp_df, shifts_df = scrape_list_of_games(games, if_scrape_shifts)

    if pbp_df is not None:
        pbp_df.to_csv('nhl_pbp{}{}.csv'.format(from_date, to_date), sep=',')
    if shifts_df is not None:
        shifts_df.to_csv('nhl_shifts{}{}.csv'.format(from_date, to_date), sep=',')

    print_errors()


def scrape_seasons(seasons, if_scrape_shifts):
    """
    Given list of seasons -> scrape all seasons 
    :param seasons: list of seasons
    :param if_scrape_shifts: if scrape shifts
    :return: nothing 
    """
    for season in seasons:
        from_date = '-'.join([str(season), '10', '1'])
        to_date = '-'.join([str(season + 1), '7', '01'])

        games = json_schedule.scrape_schedule(from_date, to_date)
        pbp_df, shifts_df = scrape_list_of_games(games, if_scrape_shifts)

        if pbp_df is not None:
            pbp_df.to_csv('nhl_pbp{}{}.csv'.format(season, season+1), sep=',')
        if shifts_df is not None:
            shifts_df.to_csv('nhl_shifts{}{}.csv'.format(season, season+1), sep=',')

    print_errors()


def scrape_games(games, if_scrape_shifts):
    """
    Scrape a list of games
    :param games: list of game_ids
    :param if_scrape_shifts: if scrape shifts 
    :return: nothing
    """
    # Create List of game_id's and dates
    games_list = json_schedule.get_dates(games)

    pbp_df, shifts_df = scrape_list_of_games(games_list, if_scrape_shifts)

    if pbp_df is not None:
        pbp_df.to_csv('nhl_pbp_games-{}.csv'.format(str(datetime.datetime.now().time())), sep=',')
    if shifts_df is not None:
        shifts_df.to_csv('nhl_shifts_games-{}.csv'.format(str(datetime.datetime.now().time())), sep=',')

    print_errors()


def print_errors():
    """
    Print errors with scraping
    """
    global broken_shifts_games
    global broken_pbp_games
    global players_missing_ids
    global espn_games

    print('\nBroken pbp:')
    for x in broken_pbp_games:
        print(x)

    print('Broken shifts:')
    for x in broken_shifts_games:
        print(x)

    print('Missing ids:')
    global players_missing_ids
    for x in players_missing_ids:
        print(x)

    print('ESPN games:')
    for x in espn_games:
        print(x)

    print('\n')

    broken_shifts_games = []
    broken_pbp_games = []
    players_missing_ids = []
    espn_games = []









import pandas as pd
from bs4 import BeautifulSoup
import time
import shared


def event_type(play_description):
    """
    Returns the event type (ex: a SHOT or a GOAL...etc) given the event description 
    :param play_description: description of play
    :return: 
    """
    events = {'GOAL SCORED': 'GOAL', 'SHOT ON GOAL': 'SHOT', 'SHOT MISSED': 'MISS', 'SHOT BLOCKED': 'BLOCK',
              'PENALTY': 'PENL', 'FACEOFF': 'FAC', 'HIT': 'HIT', 'TAKEAWAY': 'TAKE', 'GIVEAWAY': 'GIVE'}

    event = [events[e] for e in events.keys() if e in play_description]
    return event[0] if event else None


def get_espn_game_id(date, home_team, away_team):
    """
    Scrapes the day's schedule and gets the id for the given game
    Ex: http://www.espn.com/nhl/scoreboard?date=20161024
    
    :param date: format-> YearMonthDay-> 20161024
    :param home_team: home team
    :param away_team: away team
    :return: 9 digit game id
    """
    import re

    url = 'http://www.espn.com/nhl/scoreboard?date={}'.format(date.replace('-', ''))

    response = shared.get_url(url)

    regex = re.compile(r'/nhl/recap\?gameId=(\d+)')
    game_ids = regex.findall(response.text)

    # Get teams for each game
    soup = BeautifulSoup(response.content, 'lxml')
    teams = soup.findAll('td', class_='team')
    teams = [t.get_text() for t in teams if t.get_text() != '']  # Get Rid of blanks between pair of teams
    teams = [shared.TEAMS[t.upper()] for t in teams]             # Get Tricode
    games = [teams[i:i + 2] for i in range(0, len(teams), 2)]    # Make a list of both teams for each game

    for i in range(len(games)):
        if home_team in games[i] or away_team in games[i]:
            return game_ids[i]


def get_espn(date, home_team, away_team):
    """
    Gets the ESPN pbp feed 
    Ex: http://www.espn.com/nhl/gamecast/data/masterFeed?lang=en&isAll=true&gameId=400885300
    
    :param date: date of the game
    :param home_team: home team
    :param away_team: away team
    :return: raw xml
    """
    game_id = get_espn_game_id(date, home_team.upper(), away_team.upper())
    url = 'http://www.espn.com/nhl/gamecast/data/masterFeed?lang=en&isAll=true&gameId={}'.format(game_id)

    response = shared.get_url(url)
    time.sleep(1)

    return response


def parse_event(event):
    """
    Parse each event
    In the string each field is separated by a '~'. 
    Relevant for here: The first two are the x and y coordinates. And the 4th and 5th are the time elapsed and period.
    :param event: string with info
    :return: return dict with relevant info
    """
    info = dict()
    fields = event.split('~')

    # Shootouts screw everything up so don't bother...coordinates don't matter here either way
    if fields[4] == '5':
        return None

    info['xC'] = fields[0]
    info['yC'] = fields[1]
    info['time_elapsed'] = shared.convert_to_seconds(fields[3])
    info['period'] = fields[4]
    info['event'] = event_type(fields[8].upper())

    return info


def parse_espn(espn_xml):
    """
    Parse feed 
    :param espn_xml: raw xml of feed
    :return: DataFrame with info
    """
    import xml.etree.ElementTree as etree
    tree = etree.fromstring(espn_xml.text)
    events = tree[1]

    columns = ['period', 'time_elapsed', 'event', 'xC', 'yC']
    plays = [parse_event(event.text) for event in events]
    plays = [play for play in plays if play is not None]

    return pd.DataFrame(plays, columns=columns)


def scrape_game(date, home_team, away_team):
    """
    Scrape the game
    :param date: ex: 2016-20-24
    :param home_team: tricode
    :param away_team: tricode
    :return: dataframe with info 
    """
    try:
        print('Using espn for pbp')
        espn_xml = get_espn(date, home_team, away_team)
    except Exception as e:
        print('Espn pbp for game {a} {b} {c} is not there'.format(a=date, b=home_team, c=away_team), e)
        return None

    try:
        espn_df = parse_espn(espn_xml)
    except Exception as e:
        print('Error for Espn pbp for game {a} {b} {c} is not there'.format(a=date, b=home_team, c=away_team), e)
        return None

    return espn_df





import pandas as pd
from bs4 import BeautifulSoup, SoupStrainer
import re
import shared


def get_pbp(game_id):
    """
    Given a game_id it returns the raw html
    Ex: http://www.nhl.com/scores/htmlreports/20162017/PL020475.HTM
    :param game_id: the game
    :return: raw html of game
    """
    game_id = str(game_id)
    url = 'http://www.nhl.com/scores/htmlreports/{}{}/PL{}.HTM'.format(game_id[:4], int(game_id[:4]) + 1, game_id[4:])

    return shared.get_url(url)


def get_penalty(play_description):
    """
    Get the penalty info
    :param play_description: description of play field
    :return: penalty info
    """
    penalty_types = {'Instigator': 'Instigator', 'Broken stick': 'Broken stick', 'Clipping': 'Clipping',
                     'Holding the stick': 'Holding the stick', 'Slashing': 'Slashing', 'Roughing': 'Roughing',
                     'Holding': 'Holding', 'Tripping': 'Tripping', 'Hooking': 'Hooking', 'Interference': 'Interference',
                     'Cross checking': 'Cross checking', 'Boarding': 'Boarding', 'Hi-sticking': 'High-sticking',
                     'Head butting': 'Head butting', 'Cross check - double minor': 'Cross checking',
                     'Hi stick - double minor': 'High-sticking', 'Throwing stick': 'Throwing stick',
                     'Elbowing': 'Elbowing', 'Unsportsmanlike conduct': 'Unsportsmanlike conduct', 'Kneeing': 'Kneeing',
                     'Interference on goalkeeper': 'Goalie interference', 'Too many men/ice - bench': 'Too many men on ice',
                     'Illegal stick': 'Illegal stick', 'Butt ending': 'Butt-ending', 'Aggressor': 'Instigator',
                     'Puck over glass': 'Puck over glass', 'Closing hand on puck': 'Closing hand on puck',
                     'Fighting': 'Fighting', 'Spearing': 'Spearing', 'Diving': 'Diving', 'Embellishment': 'Diving',
                     'Abuse of officials': 'Abusive language', 'PS-': "Penalty Shot", 'PS -': "Penalty Shot",
                     'Illegal check to head': 'Illegal check to head', 'Goalie leave crease': 'Goalie leave crease-Delay',
                     'Charging': 'Charging', 'Delay': 'Delay of game', 'Face-off violation': 'Delay of game Faceoff',
                     'Checking from behind': 'Checking from behind', 'Illegal equipment': 'Illegal equipment',
                     'Game misconduct': 'Game misconduct', 'Game Misconduct': 'Game misconduct', 'Major': 'Major',
                     'Misconduct': 'Misconduct', 'Bench': 'bench',  'Minor': 'Minor'}

    penalty = ''
    for key in penalty_types.keys():
        if key in play_description:
            penalty = key

    if '2 min' in play_description:
        return ''.join([penalty, '(2 min)'])
    elif '4 min' in play_description:
        return ''.join([penalty, '(4 min)'])
    elif '5 min' in play_description:
        return ''.join([penalty, '(5 min)'])
    elif '10 min' in play_description:
        return ''.join([penalty, '(10 min)'])

    if penalty != '':
        return penalty
    else:
        return ''


def get_player_name(number, players, team, home_team):
    """
    This function is used for the description field in the html
    Given a last name and a number it return the player's full name and id
    :param number: player's number
    :param players: all players with info
    :param team: team of player
    :param home_team: home team
    :return: dict with full and and id
    """
    if team == home_team:
        player = [{'name': name, 'id': players['Home'][name]['id']} for name in players['Home'].keys() if
                  players['Home'][name]['number'] == number]
    else:
        player = [{'name': name, 'id': players['Away'][name]['id']} for name in players['Away'].keys() if
                  players['Away'][name]['number'] == number]

    if not player:
        player = [{'name': '', 'id': ''}]  # Control for when the name can't be found

    return player[0]


def if_valid_event(event):
    """
    Checks if it's a valid event ('#' is meaningless and I don't like the other ones) to parse
    :param event: list of stuff in pbp
    :return: boolean -True or False
    """

    if event[4] != 'GOFF' and event[0] != '#' and event[4] != 'CHL':
        return True
    else:
        return False


def return_name_html(info):
    """
    In the PBP html the name is in a format like: 'Center - MIKE RICHARDS'
    Some also have a hyphen in their last name so can't just split by '-'
    :param info: position and name
    :return: name
    """
    s = info.index('-')  # Find first hyphen
    return info[s + 1:].strip(' ')  # The name should be after the first hyphen


def shot_type(play_description):
    """
    Determine which zone the play occurred in (unless one isn't listed)
    :param play_description: the type would be in here
    :return: the type if it's there (otherwise just NA)
    """
    types = ['wrist', 'snap', 'slap', 'deflected', 'tip-in', 'backhand', 'wrap-around']

    play_description = [x.strip() for x in play_description.split(',')]  # Strip leading and trailing whitespace
    play_description = [i.lower() for i in play_description]  # Convert to lowercase

    for p in play_description:
        if p in types:
            if p == 'wrist' or p == 'slap' or p == 'snap':
                return ' '.join([p, 'shot'])
            else:
                return p
    return ''


def home_zone(event, ev_zone, ev_team, home_team):
    """
    Determines the zone relative to the home team
    :param event: event type
    :param ev_zone: zone relative to event team
    :param ev_team: event team
    :param home_team: home team
    :return: zone relative to home team
    """
    if ev_zone == '':
        return ''

    if (ev_team != home_team and event != 'BLOCK') or (ev_team == home_team and event == 'BLOCK'):
        if ev_zone == 'Off':
            return 'Def'
        elif ev_zone == 'Def':
            return 'Off'
        else:
            return ev_zone
    else:
        return ev_zone


def which_zone(play_description):
    """
    Determine which zone the play occurred in (unless one isn't listed)
    :param play_description: the zone would be included here
    :return: Off, Def, Neu, or NA
    """
    s = [x.strip() for x in play_description.split(',')]  # Split by comma's into a list
    zone = [x for x in s if 'Zone' in x]                  # Find if list contains which zone

    if not zone:
        return ''

    if zone[0].find("Off") != -1:
        return 'Off'
    elif zone[0].find("Neu") != -1:
        return 'Neu'
    elif zone[0].find("Def") != -1:
        return 'Def'


def strip_html_pbp(td):
    """
    Strip html tags and such 
    :param td: pbp
    :return: list of plays (which contain a list of info) stripped of html
    """

    for y in range(len(td)):
        # Get the 'br' tag for the time column...this get's us time remaining instead of elapsed and remaining combined
        if y == 3:
            td[y] = td[y].get_text()   # This gets us elapsed and remaining combined-< 3:0017:00
            index = td[y].find(':')
            td[y] = td[y][:index+3]
        elif (y == 6 or y == 7) and td[0] != '#':
            # 6 & 7-> These are the player one ice one's
            # The second statement controls for when it's just a header
            baz = td[y].find_all('td')
            bar = [baz[z] for z in range(len(baz)) if z % 4 != 0]  # Because of previous step we get repeats...delete some

            # The setup in the list is now: Name/Number->Position->Blank...and repeat
            # Now strip all the html
            players = []
            for i in range(len(bar)):
                if i % 3 == 0:
                    try:
                        name = return_name_html(bar[i].find('font')['title'])
                        number = bar[i].get_text().strip('\n')  # Get number and strip leading/trailing endlines
                    except KeyError:
                        name = ''
                        number = ''
                elif i % 3 == 1:
                    if name != '':
                        position = bar[i].get_text()
                        players.append([name, number, position])

            td[y] = players
        else:
            td[y] = td[y].get_text()

    return td


def clean_html_pbp(html):
    """
    Get rid of html and format the data
    :param html: the requested url
    :return: a list with all the info
    """
    strainer = SoupStrainer('td', attrs={'class': re.compile(r'bborder')})
    soup = BeautifulSoup(html.content, "lxml", parse_only=strainer)
    soup = soup.select('td.+.bborder')

    # Create a list of lists (each length 8)...corresponds to 8 columns in html pbp
    td = [soup[i:i + 8] for i in range(0, len(soup), 8)]

    cleaned_html = [strip_html_pbp(x) for x in td]

    return cleaned_html


def get_event_players(event, players, home_team):
    """
    returns a dict with the players involved in the event
    :param event: fixed up html
    :param players: dict of players and id's
    :param home_team: home team
    :return: dict with event players
    """
    info = {
        'p1_name': '',
        'p1_ID': '',
        'p2_name': '',
        'p2_ID': '',
        'p3_name': '',
        'p3_ID': '',
    }
    description = event[5].strip()
    ev_team = description.split()[0]

    if 'FAC' == event[4]:
        # MTL won Neu. Zone - MTL #11 GOMEZ vs TOR #37 BRENT
        regex = re.compile(r'(.{3})\s+#(\d+)')
        desc = regex.findall(description)  # [[Team, num], [Team, num]]

        if ev_team == desc[0][0]:
            p1 = get_player_name(desc[0][1], players, desc[0][0], home_team)
            info['p1_name'] = p1['name']
            info['p1_ID'] = p1['id']
            p2 = get_player_name(desc[1][1], players, desc[1][0], home_team)
            info['p2_name'] = p2['name']
            info['p2_ID'] = p2['id']
        else:
            p1 = get_player_name(desc[1][1], players, desc[1][0], home_team)
            info['p1_name'] = p1['name']
            info['p1_ID'] = p1['id']
            p2 = get_player_name(desc[0][1], players, desc[0][0], home_team)
            info['p2_name'] = p2['name']
            info['p2_ID'] = p2['id']

    elif event[4] in ['SHOT', 'MISS', 'GIVE', 'TAKE']:
        # MTL ONGOAL - #81 ELLER, Wrist, Off. Zone, 11 ft.
        # TOR GIVEAWAY - #35 GIGUERE, Def. Zone
        # TOR TAKEAWAY - #9 ARMSTRONG, Off. Zone

        regex = re.compile(r'#(\d+)')
        desc = regex.search(description).groups()  # num

        p = get_player_name(desc[0], players, ev_team, home_team)
        info['p1_name'] = p['name']
        info['p1_ID'] = p['id']

    elif 'HIT' == event[4]:
        # MTL #20 O'BYRNE HIT TOR #18 BROWN, Def. Zone
        regex = re.compile(r'(.{3})\s+#(\d+)')
        desc = regex.findall(description)  # [[Team, num], [Team, num]]

        p1 = get_player_name(desc[0][1], players, desc[0][0], home_team)
        info['p1_name'] = p1['name']
        info['p1_ID'] = p1['id']
        p2 = get_player_name(desc[1][1], players, desc[1][0], home_team)
        info['p2_name'] = p2['name']
        info['p2_ID'] = p2['id']

    elif 'BLOCK' == event[4]:
        # MTL #76 SUBBAN BLOCKED BY TOR #2 SCHENN, Wrist, Def. Zone

        regex = re.compile(r'(.{3})\s+#(\d+)')
        desc = regex.findall(description)  # [[Team, num], [Team, num]]

        p1 = get_player_name(desc[1][1], players, desc[1][0], home_team)
        info['p1_name'] = p1['name']
        info['p1_ID'] = p1['id']
        p2 = get_player_name(desc[0][1], players, desc[0][0], home_team)
        info['p2_name'] = p2['name']
        info['p2_ID'] = p2['id']

    elif 'GOAL' == event[4]:
        # TOR #81 KESSEL(1), Wrist, Off. Zone, 14 ft. Assists: #42 BOZAK(1); #8 KOMISAREK(1)

        regex = re.compile(r'#(\d+)\s+')
        desc = regex.findall(description)  # [num] -> ranging from 1 to 3 indices

        p1 = get_player_name(desc[0], players, ev_team, home_team)
        info['p1_name'] = p1['name']
        info['p1_ID'] = p1['id']

        if len(desc) >= 2:
            p2 = get_player_name(desc[1], players, ev_team, home_team)
            info['p2_name'] = p2['name']
            info['p2_ID'] = p2['id']

            if len(desc) == 3:
                p3 = get_player_name(desc[2], players, ev_team, home_team)
                info['p3_name'] = p3['name']
                info['p3_ID'] = p3['id']

    elif 'PENL' == event[4]:
        # MTL #81 ELLER Hooking(2 min), Def. Zone Drawn By: TOR #11 SJOSTROM
        if 'Served' in description or 'TEAM' in description:  # Check if it's a team penalty
            info['p1_name'] = 'Team'           # Since it's a team penalty
        else:
            regex = re.compile(r'(.{3})\s+#(\d+)')
            desc = regex.findall(description)  # [[team, num]] -> Either one or two indices

            p1 = get_player_name(desc[0][1], players, desc[0][0], home_team)
            info['p1_name'] = p1['name']
            info['p1_ID'] = p1['id']

            if len(desc) == 2:
                p2 = get_player_name(desc[1][1], players, desc[1][0], home_team)
                info['p2_name'] = p2['name']
                info['p2_ID'] = p2['id']

    return info


def parse_event(event, players, home_team, if_plays_in_json, current_score):
    """
    Receievs an event and parses it
    :param event: event type
    :param players: players in game
    :param home_team: home team
    :param if_plays_in_json: If the pbp json contains the plays
    :param current_score: current score for both teams
    :return: dict with info
    """
    event_dict = dict()

    away_players = event[6]
    home_players = event[7]

    try:
        event_dict['Period'] = int(event[1])
    except ValueError:
        event_dict['Period'] = 0

    event_dict['Description'] = event[5]
    event_dict['Event'] = str(event[4])

    if event_dict['Event'] in ['GOAL', 'SHOT', 'MISS', 'BLOCK', 'PENL', 'FAC', 'HIT', 'TAKE', 'GIVE']:
        event_dict['Ev_Team'] = event[5].split()[0]  # Split the description and take the first thing (which is the team)
    else:
        event_dict['Ev_Team'] = ''

    # If it's a goal change the score
    if event[4] == 'GOAL':
        if event_dict['Ev_Team'] == home_team:
            current_score['Home'] += 1
        else:
            current_score['Away'] += 1

    event_dict['Home_Score'] = current_score['Home']
    event_dict['Away_Score'] = current_score['Away']
    event_dict['score_diff'] = current_score['Home'] - current_score['Away']

    # Populate away and home player info
    for j in range(6):
        try:
            name = shared.fix_name(away_players[j][0].upper())
            event_dict['awayPlayer{}'.format(j + 1)] = name
            event_dict['awayPlayer{}_id'.format(j + 1)] = players['Away'][name]['id']
        except KeyError:
            event_dict['awayPlayer{}_id'.format(j + 1)] = 'NA'
        except IndexError:
            event_dict['awayPlayer{}'.format(j + 1)] = ''
            event_dict['awayPlayer{}_id'.format(j + 1)] = ''

        try:
            name = shared.fix_name(home_players[j][0].upper())
            event_dict['homePlayer{}'.format(j + 1)] = name
            event_dict['homePlayer{}_id'.format(j + 1)] = players['Home'][name]['id']
        except KeyError:
            event_dict['homePlayer{}_id'.format(j + 1)] = 'NA'
        except IndexError:
            event_dict['homePlayer{}'.format(j + 1)] = ''
            event_dict['homePlayer{}_id'.format(j + 1)] = ''

    # Did this because above method assumes the goalie is at end of player list
    for x in away_players:
        if x[2] == 'G':
            event_dict['Away_Goalie'] = shared.fix_name(x[0].upper())
            try:
                event_dict['Away_Goalie_Id'] = players['Away'][event_dict['Away_Goalie']]['id']
            except KeyError:
                event_dict['Away_Goalie_Id'] = 'NA'
        else:
            event_dict['Away_Goalie'] = ''
            event_dict['Away_Goalie_Id'] = ''

    for x in home_players:
        if x[2] == 'G':
            event_dict['Home_Goalie'] = shared.fix_name(x[0].upper())
            try:
                event_dict['Home_Goalie_Id'] = players['Home'][event_dict['Home_Goalie']]['id']
            except KeyError:
                event_dict['Home_Goalie_Id'] = 'NA'
        else:
            event_dict['Home_Goalie'] = ''
            event_dict['Home_Goalie_Id'] = ''

    event_dict['Away_Players'] = len(away_players)
    event_dict['Home_Players'] = len(home_players)

    try:
        home_skaters = event_dict['Home_Players'] - 1 if event_dict['Home_Goalie'] != '' else len(home_players)
        away_skaters = event_dict['Away_Players'] - 1 if event_dict['Away_Goalie'] != '' else len(away_players)
    except KeyError:
        # Getting a key error here means that home/away goalie isn't there..which means home/away players are empty
        home_skaters = 0
        away_skaters = 0

    event_dict['Strength'] = 'x'.join([str(home_skaters), str(away_skaters)])
    event_dict['Ev_Zone'] = which_zone(event[5])
    event_dict['Home_Zone'] = home_zone(event_dict['Event'], event_dict['Ev_Zone'], event_dict['Ev_Team'], home_team)

    if 'PENL' in event[4]:
        event_dict['Type'] = get_penalty(event[5])
    else:
        event_dict['Type'] = shot_type(event[5]).upper()

    event_dict['Time_Elapsed'] = str(event[3])

    if event[3] != '':
        event_dict['Seconds_Elapsed'] = shared.convert_to_seconds(event[3])
    else:
        event_dict['Seconds_Elapsed'] = ''

    # I like getting the event players from the json
    if not if_plays_in_json:
        if event_dict['Event'] in ['GOAL', 'SHOT', 'MISS', 'BLOCK', 'PENL', 'FAC', 'HIT', 'TAKE', 'GIVE']:
            event_dict.update(get_event_players(event, players, home_team))  # Add players involves in event

    return [event_dict, current_score]


def parse_html(html, players, teams, if_plays_in_json):
    """
    Parse html game pbp
    :param html: raw html
    :param players: players in the game (from json pbp)
    :param teams: dict with home and away teams
    :param if_plays_in_json: If the pbp json contains the plays
    :return: DataFrame with info
    """

    if if_plays_in_json:
        columns = ['Period', 'Event', 'Description', 'Time_Elapsed', 'Seconds_Elapsed', 'Strength', 'Ev_Zone', 'Type',
                   'Ev_Team', 'Home_Zone', 'Away_Team', 'Home_Team', 'awayPlayer1', 'awayPlayer1_id', 'awayPlayer2',
                   'awayPlayer2_id', 'awayPlayer3', 'awayPlayer3_id', 'awayPlayer4', 'awayPlayer4_id', 'awayPlayer5',
                   'awayPlayer5_id', 'awayPlayer6', 'awayPlayer6_id', 'homePlayer1', 'homePlayer1_id', 'homePlayer2',
                   'homePlayer2_id', 'homePlayer3', 'homePlayer3_id', 'homePlayer4', 'homePlayer4_id', 'homePlayer5',
                   'homePlayer5_id', 'homePlayer6', 'homePlayer6_id', 'Away_Goalie', 'Away_Goalie_Id', 'Home_Goalie',
                   'Home_Goalie_Id', 'Away_Players', 'Home_Players', 'Away_Score', 'Home_Score']
    else:
        columns = ['Period', 'Event', 'Description', 'Time_Elapsed', 'Seconds_Elapsed', 'Strength', 'Ev_Zone', 'Type',
                   'Ev_Team', 'Home_Zone', 'Away_Team', 'Home_Team', 'p1_name', 'p1_ID', 'p2_name', 'p2_ID', 'p3_name',
                   'p3_ID', 'awayPlayer1', 'awayPlayer1_id', 'awayPlayer2', 'awayPlayer2_id', 'awayPlayer3', 'awayPlayer3_id',
                   'awayPlayer4', 'awayPlayer4_id', 'awayPlayer5', 'awayPlayer5_id', 'awayPlayer6', 'awayPlayer6_id',
                   'homePlayer1', 'homePlayer1_id', 'homePlayer2', 'homePlayer2_id', 'homePlayer3', 'homePlayer3_id',
                   'homePlayer4', 'homePlayer4_id', 'homePlayer5', 'homePlayer5_id', 'homePlayer6', 'homePlayer6_id',
                   'Away_Goalie', 'Away_Goalie_Id', 'Home_Goalie', 'Home_Goalie_Id', 'Away_Players', 'Home_Players',
                   'Away_Score', 'Home_Score']

    current_score = {'Home': 0, 'Away': 0}
    events, current_score = zip(*(parse_event(event, players, teams['Home'], if_plays_in_json, current_score)
                                  for event in html if if_valid_event(event)))
    df = pd.DataFrame(list(events), columns=columns)

    df.drop(df[df.Time_Elapsed == '-16:0-'].index, inplace=True)       # This is seen sometimes...it's a duplicate row

    df['Away_Team'] = teams['Away']
    df['Home_Team'] = teams['Home']

    return df


def scrape_game(game_id, players, teams, if_plays_in_json):
    """ 
    :param game_id: game to scrape
    :param players: dict with player info
    :param teams: dict with home and away teams
    :param if_plays_in_json: boolean, if the plays are in the json
    :return: DataFrame of game info or None if it fails
    """
    try:
        game_html = get_pbp(game_id)
    except Exception as e:
        print('Problem with getting html pbp for game {}'.format(game_id), e)
        return None

    try:
        game_df = parse_html(clean_html_pbp(game_html), players, teams, if_plays_in_json)
    except Exception as e:
        print('Error for Html pbp for game {}'.format(game_id), e)
        return None

    return game_df





import time
import requests
from requests.adapters import HTTPAdapter
from requests.packages.urllib3.util.retry import Retry


"""
This file is a bunch of the shared functions or just general stuff used by the different scrapers 
in the scrape directory 
"""


TEAMS = {'ANAHEIM DUCKS': 'ANA', 'ARIZONA COYOTES': 'ARI', 'ATLANTA THRASHERS': 'ATL', 'BOSTON BRUINS': 'BOS',
         'BUFFALO SABRES': 'BUF', 'CAROLINA HURRICANES': 'CAR', 'COLUMBUS BLUE JACKETS': 'CBJ', 'CALGARY FLAMES': 'CGY',
         'CHICAGO BLACKHAWKS': 'CHI', 'COLORADO AVALANCHE': 'COL', 'DALLAS STARS': 'DAL', 'DETROIT RED WINGS': 'DET',
         'EDMONTON OILERS': 'EDM', 'FLORIDA PANTHERS': 'FLA', 'LOS ANGELES KINGS': 'L.A', 'MINNESOTA WILD': 'MIN',
         'MONTREAL CANADIENS': 'MTL', 'MONTRÉAL CANADIENS': 'MTL', 'NEW JERSEY DEVILS': 'N.J',
         'NASHVILLE PREDATORS': 'NSH', 'NEW YORK ISLANDERS': 'NYI', 'NEW YORK RANGERS': 'NYR', 'OTTAWA SENATORS': 'OTT',
         'PHILADELPHIA FLYERS': 'PHI', 'PHOENIX COYOTES': 'PHX', 'PITTSBURGH PENGUINS': 'PIT', 'SAN JOSE SHARKS': 'S.J',
         'ST. LOUIS BLUES': 'STL', 'TAMPA BAY LIGHTNING': 'T.B', 'TORONTO MAPLE LEAFS': 'TOR',
         'VANCOUVER CANUCKS': 'VAN', 'VEGAS GOLDEN KNIGHTS': 'VGK', 'WINNIPEG JETS': 'WPG', 'WASHINGTON CAPITALS': 'WSH'}


""" 
Fixes some of the mistakes made with player names

A majority of this is courtesy of Muneeb Alam (on twitter: @muneebalamcu) 
Found here -> https://github.com/muneebalam/Hockey/blob/master/NHL/Core/GetPbP.py
"""
Names = {'n/a': 'n/a', 'ALEXANDER OVECHKIN': 'Alex Ovechkin', 'TOBY ENSTROM': 'Tobias Enstrom', 'JAMIE MCGINN': 'Jamie McGinn',
         'CODY MCLEOD': 'Cody McLeod', 'MARC-EDOUARD VLASIC': 'Marc-Edouard Vlasic', 'RYAN MCDONAGH': 'Ryan McDonagh',
         'CHRIS TANEV': 'Christopher Tanev', 'JARED MCCANN': 'Jared McCann', 'P.K. SUBBAN': 'PK Subban',
         'DEVANTE SMITH-PELLY': 'Devante Smith-Pelly', 'MIKE MCKENNA': 'Mike McKenna', 'MICHAEL MCCARRON': 'Michael McCarron',
         'T.J. BRENNAN': 'TJ Brennan', 'BRAYDEN MCNABB': 'Brayden McNabb', 'PIERRE-ALEXANDRE PARENTEAU': 'PA Parenteau',
         'JAMES VAN RIEMSDYK': 'James van Riemsdyk', 'OLIVER EKMAN-LARSSON': 'Oliver Ekman-Larsson', 'TJ OSHIE': 'TJ Oshie',
         'J P DUMONT': 'JP Dumont', 'J.T. MILLER': 'JT Miller', 'R.J UMBERGER': 'RJ Umberger', 'PA PARENTEAU': 'PA Parenteau',
         'PER-JOHAN AXELSSON': 'PJ Axelsson', 'MAXIME TALBOT': 'Max Talbot', 'JOHN-MICHAEL LILES': 'John-Michael Liles',
         'DANIEL GIRARDI': 'Dan Girardi', 'DANIEL CLEARY': 'Dan Cleary', 'NIKLAS KRONVALL': 'Niklas Kronwall',
         'SIARHEI KASTSITSYN': 'Sergei Kostitsyn', 'ANDREI KASTSITSYN': 'Andrei Kostitsyn', 'ALEXEI KOVALEV': 'Alex Kovalev',
         'DAVID JOHNNY ODUYA': 'Johnny Oduya', 'EDWARD PURCELL': 'Teddy Purcell', 'NICKLAS GROSSMAN': 'Nicklas Grossmann',
         'PERNELL KARL SUBBAN': 'PK Subban', 'VOJTEK VOLSKI': 'Wojtek Wolski', 'VYACHESLAV VOYNOV': 'Slava Voynov',
         'FREDDY MODIN': 'Fredrik Modin', 'VACLAV PROSPAL': 'Vinny Prospal', 'KRISTOPHER LETANG': 'Kris Letang',
         'PIERRE ALEXANDRE PARENTEAU': 'PA Parenteau', 'T.J. OSHIE': 'TJ Oshie', 'JOHN HILLEN III': 'Jack Hillen',
         'BRANDON CROMBEEN': 'B.J. Crombeen', 'JEAN-PIERRE DUMONT': 'JP Dumont', 'RYAN NUGENT-HOPKINS': 'Ryan Nugent-Hopkins',
         'CONNOR MCDAVID': 'Connor McDavid', 'TREVOR VAN RIEMSDYK': 'Trevor van Riemsdyk', 'CALVIN DE HAAN': 'Calvin de Haan',
         'GREG MCKEGG': 'Greg McKegg', 'NATHAN MACKINNON': 'Nathan MacKinnon', 'KYLE MCLAREN': 'Kyle McLaren',
         'ADAM MCQUAID': 'Adam McQuaid', 'DYLAN MCILRATH': 'Dylan McIlrath', 'DANNY DEKEYSER': 'Danny DeKeyser',
         'JAKE MCCABE': 'Jake McCabe', 'JAMIE MCBAIN': 'Jamie McBain', 'PIERRE-MARC BOUCHARD': 'Pierre-Marc Bouchard',
         'JEAN-FRANCOIS JACQUES': 'JF Jacques', 'OLE-KRISTIAN TOLLEFSEN': 'Ole-Kristian Tollefsen',
         'MARC-ANDRE BERGERON': 'Marc-Andre Bergeron', 'MARC-ANTOINE POULIOT': 'Marc-Antoine Pouliot',
         'MARC-ANDRE GRAGNANI': 'Marc-Andre Gragnani', 'JORDAN LAVALLEE-SMOTHERMAN': 'Jordan Lavallee-Smotherman',
         'PIERRE-LUC LETOURNEAU-LEBLOND': 'Pierre Leblond', 'J-F JACQUES': 'JF Jacques', 'JP DUMONT': 'JP Dumont',
         'MARC-ANDRE CLICHE': 'Marc-Andre Cliche', 'J-P DUMONT': 'JP Dumont', 'JOSHUA BAILEY': 'Josh Bailey',
         'OLIVIER MAGNAN-GRENIER': 'Olivier Magnan-Grenier', 'FRÉDÉRIC ST-DENIS': 'Frederic St-Denis',
         'MARC-ANDRE BOURDON': 'Marc-Andre Bourdon', 'PIERRE-CEDRIC LABRIE': 'Pierre-Cedric Labrie',
         'JONATHAN AUDY-MARCHESSAULT': 'Jonathan Marchessault', 'JEAN-GABRIEL PAGEAU': 'Jean-Gabriel Pageau',
         'JEAN-PHILIPPE COTE': 'Jean-Philippe Cote', 'PIERRE-EDOUARD BELLEMARE': 'Pierre-Edouard Bellemare',
         'COLIN (JOHN) WHITE': 'Colin White', 'BATES (JON) BATTAGLIA': 'Bates Battaglia', 'MATHEW DUBMA': 'Matt Dumba',
         'NIKOLAI ANTROPOV': 'Nik Antropov', 'KRYS BARCH': 'Krystofer Barch', 'CAMERON BARKER': 'Cam Barker',
         'NICKLAS BERGFORS': 'Niclas Bergfors', 'ROBERT BLAKE': 'Rob Blake', 'MICHAEL BLUNDEN': 'Mike Blunden',
         'CHRISTOPHER BOURQUE': 'Chris Bourque', 'MICHÃ«L BOURNIVAL': 'Michael Bournival', 'NICHOLAS BOYNTON': 'Nick Boynton',
         'TJ BRENNAN': 'TJ Brennan', 'DANIEL BRIERE': 'Danny Briere', 'TJ BRODIE': 'TJ Brodie', 'J.T. BROWN': 'JT Brown',
         'ALEXANDRE BURROWS': 'Alex Burrows', 'MICHAEL CAMMALLERI': 'Mike Cammalleri', 'DANIEL CARCILLO': 'Dan Carcillo',
         'MATTHEW CARLE': 'Matt Carle', 'DANNY CLEARY': 'Dan Cleary', 'JOSEPH CORVO': 'Joe Corvo', 'JOSEPH CRABB': 'Joey Crabb',
         'BJ CROMBEEN': 'B.J. Crombeen',  'EVGENII DADONOV': 'Evgeny Dadonov', 'CHRIS VANDE VELDE': 'Chris VandeVelde',
         'JACOB DE LA ROSE': 'Jacob de la Rose', 'JOE DIPENTA': 'Joe DiPenta', 'JON DISALVATORE': 'Jon DiSalvatore',
         'JACOB DOWELL': 'Jake Dowell', 'NICHOLAS DRAZENOVIC': 'Nick Drazenovic', 'ROBERT EARL': 'Robbie Earl',
         'ALEXANDER FROLOV': 'Alex Frolov', 'T.J. GALIARDI': 'TJ Galiardi', 'TJ GALIARDI': 'TJ Galiardi',
         'ANDREW GREENE': 'Andy Greene', 'MICHAEL GRIER': 'Mike Grier', 'NATHAN GUENIN': 'Nate Guenin',
         'MARTY HAVLAT': 'Martin Havlat', 'JOSHUA HENNESSY': 'Josh Hennessy', 'T.J. HENSICK': 'TJ Hensick',
         'TJ Hensick': 'TJ Hensick', 'CHRISTOPHER HIGGINS': 'Chris Higgins', 'ROBERT HOLIK': 'Bobby Holik',
         'MATTHEW IRWIN': 'Matt Irwin', 'P. J. AXELSSON': 'PJ Axelsson', 'PER JOHAN AXELSSON': 'PJ Axelsson',
         'JONATHON KALINSKI': 'Jon Kalinski', 'ALEXANDER KHOKHLACHEV': 'Alex Khokhlachev', 'DJ KING': 'DJ King',
         'Dwayne KING': 'DJ King', 'MICHAEL KNUBLE': 'Mike Knuble', 'KRYSTOFER KOLANOS': 'Krys Kolanos',
         'MICHAEL KOMISAREK': 'Mike Komisarek', 'STAFFAN KRONVALL': 'Staffan Kronwall', 'NIKOLAY KULEMIN': 'Nikolai Kulemin',
         'CLARKE MACARTHUR': 'Clarke MacArthur', 'LANE MACDERMID': 'Lane MacDermid', 'ANDREW MACDONALD': 'Andrew MacDonald',
         'RAYMOND MACIAS': 'Ray Macias', 'CRAIG MACDONALD': 'Craig MacDonald', 'STEVE MACINTYRE': 'Steve MacIntyre',
         'MAKSIM MAYOROV': 'Maxim Mayorov', 'AARON MACKENZIE': 'Aaron MacKenzie', 'DEREK MACKENZIE': 'Derek MacKenzie',
         'RODNEY PELLEY': 'Rod Pelley', 'BRETT MACLEAN': 'Brett MacLean', 'ANDREW MACWILLIAM': 'Andrew MacWilliam',
         'BRYAN MCCABE': 'Bryan McCabe', 'OLIVIER MAGNAN': 'Olivier Magnan-Grenier', 'DEAN MCAMMOND': 'Dean McAmmond',
         'KENNDAL MCARDLE': 'Kenndal McArdle', 'ANDY MCDONALD': 'Andy McDonald', 'COLIN MCDONALD': 'Colin McDonald',
         'JOHN MCCARTHY': 'John McCarthy', 'STEVE MCCARTHY': 'Steve McCarthy', 'DARREN MCCARTY': 'Darren McCarty',
         'JAY MCCLEMENT': 'Jay McClement', 'CODY MCCORMICK': 'Cody McCormick', 'MAX MCCORMICK': 'Max McCormick',
         'BROCK MCGINN': 'Brock McGinn', 'TYE MCGINN': 'Tye McGinn', 'BRIAN MCGRATTAN': 'Brian McGrattan',
         'DAVID MCINTYRE': 'David McIntyre', 'NATHAN MCIVER': 'Nathan McIver', 'JAY MCKEE': 'Jay McKee',
         'CURTIS MCKENZIE': 'Curtis McKenzie', 'FRAZER MCLAREN': 'Frazer McLaren', 'BRETT MCLEAN': 'Brett McLean',
         'BRANDON MCMILLAN': 'Brandon McMillan', 'CARSON MCMILLAN': 'Carson McMillan', 'PHILIP MCRAE':
         'Philip McRae', 'FREDERICK MEYER IV': 'Freddy Meyer', 'MICHAEL MODANO': 'Mike Modano',
         'CHRISTOPHER NEIL': 'Chris Neil', 'MATTHEW NIETO': 'Matt Nieto', 'JOHN ODUYA': 'Johnny Oduya',
         'PIERRE PARENTEAU': 'Pierre Parenteau', 'MARC POULIOT': 'Marc-Antoine Pouliot', 'MAXWELL REINHART': 'Max Reinhart',
         'MICHAEL RUPP': 'Mike Rupp', 'ROBERT SCUDERI': 'Rob Scuderi', 'TOMMY SESTITO': 'Tom Sestito',
         'MICHAEL SILLINGER': 'Mike Sillinger', 'JONATHAN SIM': 'Jon Sim', 'MARTIN ST LOUIS': 'Martin St. Louis',
         'MATTHEW STAJAN': 'Matt Stajan', 'ZACHERY STORTINI': 'Zack Stortini', 'PK SUBBAN': 'PK Subban',
         'WILLIAM THOMAS': 'Bill Thomas', 'R.J. UMBERGER': 'RJ Umberger', 'RJ UMBERGER': 'RJ Umberger',
         'MARK VAN GUILDER': 'Mark van Guilder', 'BRYCE VAN BRABANT': 'Bryce van Brabant',
         'DAVID VAN DER GULIK': 'David van der Gulik', 'MIKE VAN RYN': 'Mike van Ryn', 'ANDREW WOZNIEWSKI': 'Andy Wozniewski',
         'JAMES WYMAN': 'JT Wyman', 'JT WYMAN': 'JT Wyman', 'NIKOLAY ZHERDEV': 'Nikolai Zherdev',
         'HARRISON ZOLNIERCZYK': 'Harry Zolnierczyk', 'MARTIN ST PIERRE': 'Martin St. Pierre',
         'DENIS GAUTHIER JR.': 'Denis Gauthier Jr.', 'MARC-ANDRE FLEURY': 'Marc-Andre Fleury', 'DAN LACOUTURE': 'Dan LaCouture',
         'RICK DIPIETRO': 'Rick DiPietro', 'JOEY MACDONALD': 'Joey MacDonald', 'B.J CROMBEEN': 'B.J. Crombeen',
         'TIMOTHY JR. THOMAS': 'Tim Thomas', 'ILJA BRYZGALOV': 'Ilya Bryzgalov', 'MATHEW DUMBA': 'Matt Dumba',
         'MICHAËL BOURNIVAL': 'Michael Bournival', 'MATTHEW BENNING': 'Matt Benning', 'ZACHARY SANFORD': 'Zach Sanford',
         'AJ GREER': 'A.J. Greer', 'JT COMPHER': 'J.T. Compher', 'NICOLAS PETAN': 'Nic Petan',
         'VINCENT HINOSTROZA': 'Vinnie  Hinostroza', 'PHILIP VARONE': 'Phil Varone', 'JOSHUA MORRISSEY': 'Josh Morrissey',
         'Mathew Bodie': 'Mat Bodie', 'MICHAEL FERLAND': 'Micheal Ferland', 'MICHAEL SANTORELLI' :'Mike Santorelli',
         'CHRISTOPHER BREEN': 'Chris Breen', 'BRYCE VAN BRABRANT': 'Bryce Van Brabant', 'ALEXANDER KILLORN': 'Alex Killorn',
         'JOSEPH MORROW': 'Joe Morrow', 'ALEX STEEN': 'Alexander Steen', 'BRADLEY MILLS': 'Brad Mills',
         'MICHAEL SISLO': 'Mike Sislo', 'MICHAEL VERNACE': 'Mike Vernace', 'STEVEN REINPRECHT': 'Steve Reinprecht',
         'MATTHEW MURRAY': 'Matt Murray', 'THOMAS MCCOLLUM': 'TOM MCCOLLUM', 'MICHAEL MATHESON': 'MIKE MATHESON'}


def fix_name(name):
    """
    Check if a name falls under those that need fixing.
    If it does...fix it
    :param name: name in pbp
    :return: Either the given parameter or the fixed name
    """
    if name in Names.keys():
        return Names[name].upper()
    else:
        return name


def convert_to_seconds(minutes):
    """
    Return minutes remaining in time format to seconds elapsed
    :param minutes: time remaining
    :return: time elapsed in seconds
    """
    if minutes == '-16:0-':
        return '1200'      # Sometimes in the html at the end of the game the time is -16:0-

    import datetime
    x = time.strptime(minutes.strip(' '), '%M:%S')

    return datetime.timedelta(hours=x.tm_hour, minutes=x.tm_min, seconds=x.tm_sec).total_seconds()


def get_url(url):
    """
    Get the url
    :param url: given url
    :return: page
    """
    response = requests.Session()
    retries = Retry(total=10, backoff_factor=.1)
    response.mount('http://', HTTPAdapter(max_retries=retries))

    response = response.get(url, timeout=5)
    response.raise_for_status()

    return response









import pandas as pd
import requests
import json
import time
import shared


def get_pbp(game_id):
    """
    Given a game_id it returns the raw json
    Ex: http://statsapi.web.nhl.com/api/v1/game/2016020475/feed/live
    :param game_id: the game
    :return: raw json of game
    """
    url = 'http://statsapi.web.nhl.com/api/v1/game/{}/feed/live'.format(game_id)

    try:
        response = shared.get_url(url)
        pbp_json = json.loads(response.text)
    except requests.exceptions.HTTPError as e:
        print('Json pbp for game {} is not there'.format(game_id), e)
        return None

    time.sleep(1)
    return pbp_json


def change_event_name(event):
    """
    Change event names from json style to html
    ex: BLOCKED_SHOT to BLOCK
    :param event: event type
    :return: fixed event type
    """
    event_types ={
        'PERIOD_START': 'PSTR',
        'FACEOFF': 'FAC',
        'BLOCKED_SHOT': 'BLOCK',
        'GAME_END': 'GEND',
        'GIVEAWAY': 'GIVE',
        'GOAL': 'GOAL',
        'HIT': 'HIT',
        'MISSED_SHOT': 'MISS',
        'PERIOD_END': 'PEND',
        'SHOT': 'SHOT',
        'STOP': 'STOP',
        'TAKEAWAY': 'TAKE',
        'PENALTY': 'PENL',
        'Early Intermission Start': 'EISTR',
        'Early Intermission End': 'EIEND',
        'Shootout Completed': 'SOC',
    }

    try:
        return event_types[event]
    except KeyError:
        return event


def get_teams(json):
    """
    Get teams 
    :param json: pbp json
    :return: dict with home and away
    """
    return {'Home': shared.TEAMS[json['gameData']['teams']['home']['name'].upper()],
            'Away': shared.TEAMS[json['gameData']['teams']['away']['name'].upper()]}


def parse_event(event):
    """
    Parses a single event when the info is in a json format
    :param event: json of event 
    :return: dictionary with the info
    """
    play = dict()

    play['period'] = event['about']['period']
    play['event'] = str(change_event_name(event['result']['eventTypeId']))
    play['seconds_elapsed'] = shared.convert_to_seconds(event['about']['periodTime'])

    # If there's a players key that means an event occurred on the play.
    if 'players' in event.keys():
        play['p1_name'] = shared.fix_name(event['players'][0]['player']['fullName'])
        play['p1_ID'] = event['players'][0]['player']['id']

        for i in range(len(event['players'])):
            if event['players'][i]['playerType'] != 'Goalie':
                play['p{}_name'.format(i + 1)] = shared.fix_name(event['players'][i]['player']['fullName'].upper())
                play['p{}_ID'.format(i + 1)] = event['players'][i]['player']['id']

        # Coordinates aren't always there
        try:
            play['xC'] = event['coordinates']['x']
            play['yC'] = event['coordinates']['y']
        except KeyError:
            play['xC'] = ''
            play['yC'] = ''

    """
    # Sometimes they record events for shots in the wrong zone (or maybe not)...so change it
    if play['xC'] != 'Na' and play['yC'] != 'Na':

        if play['Ev_Team'] == home_team:
            # X should be negative in 1st and 3rd for home_team
            if (play['Period'] == 1 or play['Period'] == 3) and play['xC'] > 0:
                play['xC'] = -int(play['xC'])
                play['yC'] = -int(play['yC'])
            elif play['Period'] == 2 and play['xC'] < 0:
                play['xC'] = -int(play['xC'])
                play['yC'] = -int(play['yC'])
        else:
            # X should be positive in 1st and 3rd for away_team
            if (play['Period'] == 1 or play['Period'] == 3) and play['xC'] < 0:
                play['xC'] = -int(play['xC'])
                play['yC'] = -int(play['yC'])
            elif play['Period'] == 2 and play['xC'] > 0:
                play['xC'] = -int(play['xC'])
                play['yC'] = -int(play['yC'])
    """

    return play


def parse_json(game_json):
    """
    Scrape the json for a game
    :param game_json: raw json
    :return: Either a DataFrame with info for the game 
    """
    columns = ['period', 'event', 'seconds_elapsed', 'p1_name', 'p1_ID', 'p2_name', 'p2_ID', 'p3_name', 'p3_ID', 'xC', 'yC']

    plays = game_json['liveData']['plays']['allPlays'][2:]  # All the plays/events in a game

    # Go through all events and store all the info in a list
    # 'PERIOD READY' & 'PERIOD OFFICIAL'..etc aren't found in html...so get rid of them
    event_to_ignore = ['PERIOD_READY', 'PERIOD_OFFICIAL', 'GAME_READY', 'GAME_OFFICIAL']
    events = [parse_event(play) for play in plays if play['result']['eventTypeId'] not in event_to_ignore]

    return pd.DataFrame(events, columns=columns)


def scrape_game(game_id):
    """
    Used for debugging 
    :param game_id: game to scrape
    :return: DataFrame of game info
    """
    try:
        game_json = get_pbp(game_id)
    except requests.exceptions.HTTPError as e:
        print('Json pbp for game {} is not there'.format(game_id), e)
        return None

    try:
        game_df = parse_json(game_json)
    except Exception as e:
        print('Error for Json pbp for game {}'.format(game_id), e)
        return None

    return game_df




from bs4 import BeautifulSoup
import shared


def get_roster(game_id):
    """
    Given a game_id it returns the raw html
    Ex: http://www.nhl.com/scores/htmlreports/20162017/RO020475.HTM
    :param game_id: the game
    :return: raw html of game
    """
    game_id = str(game_id)
    url = 'http://www.nhl.com/scores/htmlreports/{}{}/RO{}.HTM'.format(game_id[:4], int(game_id[:4]) + 1, game_id[4:])

    return shared.get_url(url)


def fix_name(player):
    """
    Sometimes a player had a (A) or (C) attached to their name
    :param player: list of player info -> [number, position, name]
    :return: fixed list
    """
    player[2] = player[2][:player[2].find('(')]
    player[2] = player[2].strip()

    return player


def get_coaches(soup):
    """
    scrape head coaches
    :param soup: html
    :return: dict of coaches for game
    """
    head_coaches = dict()

    coaches = soup.find_all('tr', {'id': "HeadCoaches"})
    coaches = coaches[0].find_all('td')
    head_coaches['Away'] = coaches[1].get_text()
    head_coaches['Home'] = coaches[3].get_text()

    return head_coaches


def get_players(soup):
    """
    scrape roster for players 
    :param soup: html
    :return: dict for home and away players
    """
    players = dict()

    tables = soup.findAll('table', {'align': 'center', 'border': '0', 'cellpadding': '0', 'cellspacing': '0', 'width': '100%'})
    """
    There are 5 tables which correspond to the above criteria.
    tables[0] is nothing
    tables[1] is away starters
    tables[2] is home starters
    tables[3] is away scratches
    tables[4] is home scratches
    """

    away_starters = tables[1].find_all('td')
    away_scratches = tables[3].find_all('td')
    home_starters = tables[2].find_all('td')
    home_scratches = tables[4].find_all('td')

    away_starters = [i.get_text() for i in away_starters]
    away_scratches = [i.get_text() for i in away_scratches]
    home_starters = [i.get_text() for i in home_starters]
    home_scratches = [i.get_text() for i in home_scratches]

    # Make list of list of 3 each. The three are: number, position, name (in that order)
    away_starters = [away_starters[i:i + 3] for i in range(0, len(away_starters), 3)]
    away_scratches = [away_scratches[i:i + 3] for i in range(0, len(away_scratches), 3)]
    home_starters = [home_starters[i:i + 3] for i in range(0, len(home_starters), 3)]
    home_scratches = [home_scratches[i:i + 3] for i in range(0, len(home_scratches), 3)]

    # Get rid of header column
    away_starters = [i for i in away_starters if i[0] != '#']
    away_scratches = [i for i in away_scratches if i[0] != '#']
    home_starters = [i for i in home_starters if i[0] != '#']
    home_scratches = [i for i in home_scratches if i[0] != '#']

    away_players = away_starters + away_scratches
    home_players = home_starters + home_scratches

    # For those with (A) or (C) in name field get rid of it
    # first condition is to control when we get whitespace as one of the indices
    players['Away'] = [fix_name(i) if i[0] != '\xa0' and i[2].find('(') != -1 else i for i in away_players]
    players['Home'] = [fix_name(i) if i[0] != '\xa0' and i[2].find('(') != -1 else i for i in home_players]

    # Get rid when just whitespace
    players['Away'] = [i for i in away_players if i[0] != '\xa0']
    players['Home'] = [i for i in home_players if i[0] != '\xa0']

    return players


def scrape_roster(game_id):
    """
    For a given game scrapes the roster
    :param game_id: id for game
    :return: dict of players (home and away) an dict for both head coaches 
    """

    try:
        roster = get_roster(game_id)
    except Exception as e:
        print('Roster for game {} is not there'.format(game_id), e)
        raise Exception

    try:
        soup = BeautifulSoup(roster.content, "html.parser")
        players = get_players(soup)
        head_coaches = get_coaches(soup)
    except Exception as e:
        print('Problem with playing roster for game {}'.format(game_id), e)
        raise Exception

    return {'players': players, 'head_coaches': head_coaches}





import pandas as pd
from bs4 import BeautifulSoup
import time
import shared


def get_teams(soup):
    """
    Return the team for the TOI tables and the home team
    :param soup: souped up html
    :return: list with team and home team
    """
    import re

    team = soup.find('td', class_='teamHeading + border')  # Team for shifts
    team = team.get_text()

    # Get Home Team
    teams = soup.find_all('td', {'align': 'center', 'style': 'font-size: 10px;font-weight:bold'})
    regex = re.compile(r'>(.*)<br/>')
    home_team = regex.findall(str(teams[7]))

    return [team, home_team[0]]


def analyze_shifts(shift, name, team, home_team, player_ids):
    """
    Analyze shifts for each player when using.
    Prior to this each player (in a dictionary) has a list with each entry being a shift.
    This function is only used for the html
    :param shift: info on shift
    :param name: player name
    :param team: given team
    :param home_team: home team for given game
    :param player_ids: dict with info on players
    :return: dict with info for shift
    """
    shifts = dict()

    shifts['Player'] = name.upper()
    shifts['Period'] = '4' if shift[1] == 'OT' else shift[1]
    shifts['Team'] = shared.TEAMS[team.strip(' ')]
    shifts['Start'] = shared.convert_to_seconds(shift[2].split('/')[0])
    shifts['End'] = shared.convert_to_seconds(shift[3].split('/')[0])
    shifts['Duration'] = shared.convert_to_seconds(shift[4].split('/')[0])

    try:
        if home_team == team:
            shifts['Player_Id'] = player_ids['Home'][name.upper()]['id']
        else:
            shifts['Player_Id'] = player_ids['Away'][name.upper()]['id']
    except KeyError:
        shifts['Player_Id'] = ''

    return shifts


def get_shifts(game_id):
    """
    Given a game_id it returns a DataFrame with the shifts for both teams
    Ex: http://www.nhl.com/scores/htmlreports/20162017/TV020971.HTM
    :param game_id: the game
    :return: DataFrame with all shifts, return None when an exception is thrown when parsing
    """
    game_id = str(game_id)
    home_url = 'http://www.nhl.com/scores/htmlreports/{}{}/TH{}.HTM'.format(game_id[:4], int(game_id[:4])+1, game_id[4:])
    away_url = 'http://www.nhl.com/scores/htmlreports/{}{}/TV{}.HTM'.format(game_id[:4], int(game_id[:4])+1, game_id[4:])

    home = shared.get_url(home_url)
    time.sleep(1)

    away = shared.get_url(away_url)
    time.sleep(1)

    return home, away


def parse_html(html, player_ids, game_id):
    """
    Parse the html
    :param html: cleaned up html
    :param player_ids: dict of home and away players
    :param game_id: id for game
    :return: DataFrame with info
    """
    columns = ['Game_Id', 'Player', 'Player_Id', 'Period', 'Team', 'Start', 'End', 'Duration']
    df = pd.DataFrame(columns=columns)

    soup = BeautifulSoup(html.content, "lxml")

    teams = get_teams(soup)
    team = teams[0]
    home_team = teams[1]

    td = soup.findAll(True, {'class': ['playerHeading + border', 'lborder + bborder']})

    """
    The list 'td' is laid out with player name followed by every component of each shift. Each shift contains: 
    shift #, Period, begin, end, and duration. The shift event isn't included. 
    """
    players = dict()
    for t in td:
        t = t.get_text()
        if ',' in t:     # If it has a comma in it we know it's a player's name...so add player to dict
            name = t
            # Just format the name normally...it's coded as: 'num last_name, first_name'
            name = name.split(',')
            name = ' '.join([name[1].strip(' '), name[0][2:].strip(' ')])
            name = shared.fix_name(name)
            players[name] = dict()
            players[name]['number'] = name[0][:2].strip()
            players[name]['Shifts'] = []
        else:
            # Here we add all the shifts to whatever player we are up to
            players[name]['Shifts'].extend([t])

    for key in players.keys():
        # Create a list of lists (each length 5)...corresponds to 5 columns in html shifts
        players[key]['Shifts'] = [players[key]['Shifts'][i:i + 5] for i in range(0, len(players[key]['Shifts']), 5)]

        shifts = [analyze_shifts(shift, key, team, home_team, player_ids) for shift in players[key]['Shifts']]
        df = df.append(shifts, ignore_index=True)

    df['Game_Id'] = str(game_id)[5:]
    return df


def scrape_game(game_id, players):
    """
    Scrape the game.
    Try the json first, if it's not there do the html (it should be there for all games)
    :param game_id: id for game
    :param players: list of players
    :return: DataFrame with info for the game
    """
    home_html, away_html = get_shifts(game_id)

    away_df = parse_html(away_html, players, game_id)
    home_df = parse_html(home_html, players, game_id)

    game_df = pd.concat([away_df, home_df], ignore_index=True)

    game_df = game_df.sort_values(by=['Period', 'Start'], ascending=[True, True])  # Sort by period and by time
    game_df = game_df.reset_index(drop=True)

    return game_df






import os
from setuptools import setup


def read():
    return open(os.path.join(os.path.dirname(__file__), 'README.rst')).read()

setup(
    name='hockey_scraper',
    version='1',
    description="""This project is designed to allow people to scrape Play by Play and Shift data off of the National
                Hockey League (NHL) API and website for all regular season and playoff games since the 2010-2011 
                season""",
    long_description=read(),
    classifiers=[
        "Development Status :: 4 - Beta",
        'Intended Audience :: Science/Research',
        "License :: OSI Approved :: MIT License",
        'Programming Language :: Python :: 3'
    ],
    keywords='NHL',
    url='https://github.com/HarryShomer/Hockey-Scraper',
    author='Harry Shomer',
    author_email='Harryshomer@gmail.com',
    license='MIT',
    packages=['scrape'],
    install_requires=['pandas', 'BeautifulSoup4', 'requests', 'lxml'],
    include_package_data=True,
    zip_safe=False
)



import pandas as pd
import json
import time
import shared


def get_shifts(game_id):
    """
    Given a game_id it returns the raw json
    Ex: http://www.nhl.com/stats/rest/shiftcharts?cayenneExp=gameId=2010020001
    :param game_id: the game
    :return: 
    """
    url = 'http://www.nhl.com/stats/rest/shiftcharts?cayenneExp=gameId={}'.format(game_id)
    response = shared.get_url(url)
    time.sleep(1)

    shift_json = json.loads(response.text)
    return parse_json(shift_json, game_id)


def fix_team_tricode(tricode):
    """
    Some of the tricodes are different than how I want them
    :param tricode: 3 letter team name - ex: NYR
    :return: fixed tricode
    """
    fixed_tricodes = {
        'TBL':  'T.B',
        'LAK': 'L.A',
        'NJD': 'N.J',
        'SJS': 'S.J'
    }
    if tricode.upper() in list(fixed_tricodes.keys()):
        return fixed_tricodes[tricode.upper()]
    else:
        return tricode


def parse_shift(shift):
    """
    Parse shift for json
    :param shift: json for shift
    :return: dict with shift info
    """
    shift_dict = dict()
    name = shared.fix_name(' '.join([shift['firstName'].strip(' ').upper(), shift['lastName'].strip(' ').upper()]))
    shift_dict['Player'] = name
    shift_dict['Player_Id'] = shift['playerId']
    shift_dict['Period'] = shift['period']

    shift_dict['Team'] = fix_team_tricode(shift['teamAbbrev'])

    # At the end of the json they list when all the goal events happened. They are the only one's which have their
    # eventDescription be not null
    if shift['eventDescription'] is None:
        shift_dict['Start'] = shared.convert_to_seconds(shift['startTime'])
        shift_dict['End'] = shared.convert_to_seconds(shift['endTime'])
        shift_dict['Duration'] = shared.convert_to_seconds(shift['duration'])
    else:
        shift_dict = dict()

    return shift_dict


def parse_json(shift_json, game_id):
    """
    Parse the json
    :param shift_json: raw json
    :param game_id: if of game
    :return: DataFrame with info
    """
    columns = ['Game_Id', 'Period', 'Team', 'Player', 'Player_Id', 'Start', 'End', 'Duration']

    shifts = [parse_shift(shift) for shift in shift_json['data']]     # Go through the shifts
    shifts = [shift for shift in shifts if shift != {}]               # Get rid of null shifts (which happen at end)

    df = pd.DataFrame(shifts, columns=columns)
    df['Game_Id'] = str(game_id)[5:]
    df = df.sort_values(by=['Period', 'Start'], ascending=[True, True])  # Sort by period by time
    df = df.reset_index(drop=True)

    return df


def scrape_game(game_id):
    """
    Scrape the game.
    Try the json first, if it's not there do the html (it should be there for all games)
    :param game_id: game
    :return: DataFrame with info for the game
    """
    game_df = get_shifts(game_id)

    return game_df






import json
import time
import shared


def get_schedule(date_from, date_to):
    """
   Scrapes games in date range
    Ex: https://statsapi.web.nhl.com/api/v1/schedule?startDate=2010-10-03&endDate=2011-06-20
    :param date_from: scrape from this date
    :param date_to: scrape until this date
    :return: raw json of schedule of date range
    2010-10-03
    """
    url = 'https://statsapi.web.nhl.com/api/v1/schedule?startDate={a}&endDate={b}'.format(a=date_from, b=date_to)

    response = shared.get_url(url)
    time.sleep(1)

    schedule_json = json.loads(response.text)

    return schedule_json


def get_dates(games):
    """
    Given a game_id returns the date of the game
    :param games: ex: 2016020001
    :return: date of game (empty string if can't find it)
    """
    games.sort()

    year_from = str(games[0])[:4]
    year_to = str(games[len(games)-1])[:4]
    date_from = '-'.join([year_from, '10', '1'])             # Earliest games in sample
    date_to = '-'.join([str(int(year_to)+1), '6', '25'])    # Newest game in sample

    schedule = scrape_schedule(date_from, date_to)

    games_list = []
    for game in schedule:
        if game[0] in games:
            games_list.extend([game])

    return games_list


def scrape_schedule(date_from, date_to):
    """
    Calls getSchedule and scrapes the raw schedule JSON
    :param date_from: scrape from this date
    :param date_to: scrape until this date
    :return: list with all the game id's
    """
    schedule = []

    schedule_json = get_schedule(date_from, date_to)
    for day in schedule_json['dates']:
        for game in day['games']:
            # Don't bother with preseason
            if int(str(game['gamePk'])[5:]) >= 20000:
                # schedule[str(game['gamePk'])] = day['date']
                schedule.append([game['gamePk'], day['date']])

    return schedule

